# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      class HousingChannelsController < Hermes::Api::Dmo::ApplicationController
        before_action :authorize_administrator! , except: [:show, :index]
        set_instance_for %i[index show create update destroy]

        class << self
          def mutate_params
            {
              key: {type: 'Sting'},
              label: {type: 'Sting'},
              request_name: {type: 'Sting'},
              code_ikos: {type: 'Sting'},
            }
          end 
      
          def acceptable_params
            {

            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $eq $all]
          end
        end

        def index
          acceptable_params *self.class.acceptable_params.keys
          valid_operators *self.class.valid_operators
          paginate(
            index_aggregation(no_acl: true),
            each_serializer: @serializer,
            **@options
          )
        end

        def show 
          @instance = @model.find(params[:id])
          super
        end 

        def authorize_administrator!
          return render_json_error(:unauthorized, :not_admin) unless current_user_roles.include?('Administrator') || is_super_admin
        end
      end
    end
  end
end