# frozen_string_literal: true

NotificationCenter::Subscription.class_eval do
  before_save :before_save_actions
  PROVIDERS = [:email, :web, :slack, :sms]

  index({ subscriber_id: 1}, { background: true })
  index({ subscriber_type: 1}, { background: true })

  def init
    default_keys = self.class.keys(subscriber) # ['message.new']
    default_keys = default_keys.map { |key| default_key_values(key) }
    self.update(keys: default_keys)
  end

  def before_save_actions
    update_keys_updated_at
  end 
  
  def update_keys_updated_at
    keys.each do |k|
      old_key = keys_was&.detect { |a| a[:name] == k[:name] }
      if !old_key || PROVIDERS.any? { |p| !old_key[p] || k[p] != old_key[p] }
        us = Hermes::UpdateSubscription.find_or_create_by(name: k[:name], object_id: subscriber_id, reference_model_name: subscriber_type )
        us.update(email: k[:email], web: k[:web], slack: k[:slack], sms: k[:sms])
      end 
    end 
  end 

  private

  def default_key_values(key)
    { name: key, email: true, web: true, slack: false, sms: false, updated_at: DateTime.current }
  end 

  def update_subscription(key, provider, value)
    new_keys = keys
    if new_keys.map { |key| key[:name] }.exclude?(key)
      new_keys.push(default_key_values(key))
    end

    new_keys = new_keys.map do |k|
      if k[:name] == key
        k[provider] = value
        k[:updated_at] = DateTime.current
        k
      else
        k
      end
    end

    self.update({ keys: new_keys })
  end
end
