require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  
describe Hermes::Api::Dmo::LessorObjectiveHistoriesController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:user) {create(:user)}
  let(:objective) do
    create(
      :lessor_objective,
      target_by_typologies: { T2: 3 },
      realized_by_typologies: { T2: 1 }
    )
  end
  let(:history) { build(:lessor_objective_history, lessor_objective: objective) }
  #TESTS BASIQUES D'API
  
  
  describe 'GET /lessor_objective_historyes' do
    describe 'GET /lessor_objective_histories' do
      let!(:lessor_objective_history1) {create(:lessor_objective_history,  lessor_objective: objective)}
      let!(:lessor_objective_history3) {create(:lessor_objective_history,  lessor_objective: objective)}
      let!(:lessor_objective_history2) {create(:lessor_objective_history,  lessor_objective: objective)}
      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end
      it 'returns lessor_objective_history code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end
  end
  
  describe 'GET /lessor_objective_historyes/:id' do
    describe 'verif it work' do
      let!(:lessor_objective_history1) {create(:lessor_objective_history,  lessor_objective: objective)}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: lessor_objective_history1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::LessorObjectiveHistoriesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_LESSOR_OBJECTIVE_HISTORIES'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
  let(:objective) do
    create(
      :lessor_objective,
      target_by_typologies: { T2: 3 },
      realized_by_typologies: { T2: 1 }
    )
  end
  context 'as admin' do
    let!(:admin) {create(:administrator)}
    
    path '/api/dmo/lessor_objective_histories' do
      get 'Retrieves all lessor_objective_histories' do
        tags tags
        produces 'application/json'
        
        Hermes::Api::Dmo::LessorObjectiveHistoriesController.acceptable_params.sort.each do |params_name, options|
          parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
        end
        
        security [Bearer: {}]
        paginator
        response '200', 'lessor_objective_histories found' do
          let(:Authorization) {"Bearer #{token_for(admin)}"}
          let!(:lessor_objective_histories) {create_list(:lessor_objective_history, 5,  lessor_objective: objective)}
          include_context 'with integration test'
          include_context 'valid jsonapi collection' do
            let(:expected_collection) {lessor_objective_histories}
          end
        end
      end
    end
    
    path '/api/dmo/lessor_objective_histories/{id}' do
      get 'Retrieves a single lessor_objective_history by id' do
        tags tags
        produces 'application/json'
        parameter name: :id, in: :path, type: :string
        security [Bearer: {}]
        
        response '200', 'lessor_objective_history found' do
          let(:Authorization) {"Bearer #{token_for(admin)}"}
          
          let(:lessor_objective_history) {create(:lessor_objective_history, lessor_objective: objective)}
          
          let(:id) {lessor_objective_history.id.to_s}
          
          include_context 'with integration test'
          include_context 'valid jsonapi ressource' do
            let(:expected_ressource) {lessor_objective_history}
          end
        end
      end
    end
  end
end