require 'rails_helper'

RSpec.describe ExecuteFlux36TaskJob do
  describe 'perform' do
    subject { ExecuteFlux36TaskJob.perform_now(hr.id.to_s) }
    let!(:hr) { create(:complete_request)}
    let!(:attributes) { { als_id: '1', entity_id: '1', first_flux_30_sended: true}}
  
    before do 
      hr.set(attributes)
    end 
  
    context 'default' do 
      it { expect(subject.class).to eq(ExecuteSopraTaskJob) }
    end 
  
    context 'when entity_id nil' do 
      let!(:attributes) { { als_id: '1', first_flux_30_sended: true}}
      it { expect(subject.class).to eq(ExecuteSopraTaskJob) }
    end 
  
    context 'generate error' do 
      let!(:attributes) { {}}
  
      it { expect(subject.class).to eq(ExecuteFlux36TaskJob) }
  
      it 'create error when max try' do 
        expect(Log.where(code: 'FLUX_36_ERROR_MISSING_ATTRIBUTES').count).to eq(0)
    
        ExecuteFlux36TaskJob.perform_later(hr.id.to_s)
        (0..5).each do |i|
          last_job = if i == 0
            enqueued_jobs.select{|a| a[:job] == ExecuteFlux36TaskJob }.first
          else
            enqueued_jobs.select{|a| a[:job] == ExecuteFlux36TaskJob && a[:args][1] == i }.first
          end 
          last_job[:job].perform_now(*deserialize_argument(last_job[:args]))
        end 
        Hermes::LogJob.perform_now
        expect(Log.where(code: 'FLUX_36_ERROR_WRONG_ATTRIBUTES').count).to eq(1)
      end 
    end 
  end 

  describe 'triggered by' do
    let!(:entity) { create(:entity) }
    let!(:entity_code) { create(:entity_code, entity_id: entity.id, code: 'CODE') }
    let!(:hr) { create(:complete_request, als_id: '1', first_flux_30_sended: true, last_entity_code: entity_code.code ) }
    let(:attributes) { {} }
  
    subject { hr.reload.update(attributes) }
  
    before { enqueued_jobs.clear; ExecuteFlux36TaskJob.unlock_all }

    context 'no update' do
      it { expect { subject }.not_to have_enqueued_job(ExecuteFlux36TaskJob)}
    end
  
    context 'when first_flux_30_sended become true' do
      before { hr.set(first_flux_30_sended: false) }
      let(:attributes) { {first_flux_30_sended: true} }

      it { expect { subject }.to have_enqueued_job(ExecuteFlux36TaskJob)}
    end

    context 'when consent_level 1' do
      let(:attributes) { {consent_level: 1} }

      it { subject;expect(hr.reload.entity_id).to be_nil }
      it { expect { subject }.to have_enqueued_job(ExecuteFlux36TaskJob)}
    end 

    context 'when entity_id changed' do
      let(:attributes) { {entity_id: "2"} }

      it { expect { subject }.to have_enqueued_job(ExecuteFlux36TaskJob)}
    end 

    context 'when entity_id become nil' do
      let(:attributes) { {entity_id: nil} }

      it { expect { subject }.to have_enqueued_job(ExecuteFlux36TaskJob)}
      context 'and entity_id already nil' do
        before { hr.set(entity_id: nil) }
        
        it { expect { subject }.not_to have_enqueued_job(ExecuteFlux36TaskJob)}
      end 
    end 
  end 
end
