require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')


RSpec.describe UpdateAsyncEligibleNbOfferJob do
  subject { UpdateAsyncEligibleNbOfferJob }
  let!(:offer) { create(:housing_offer, :with_eligible_requests, requests_count: 6 )}

  it 'should create jobs' do
    enqueued_jobs.clear
    UpdateAsyncEligibleNbOfferJob.unlock_all
    expect do
      offer.update(offer_status: 'EN ATTENTE COMMERCIALISATION')
    end.to change { enqueued_jobs.select{|job| job[:job] == subject }.length }.from(0).to(1)
  end

  it 'update fields' do 
    offer.eligibles_requests.each_with_index do |hr, i|
      hr.set(seeked_district_insee_codes: [], bordering_insee_codes: [], all_eligibility_codes: []) if [0].include?(i)
      hr.set(seeked_district_insee_codes: [offer.insee_code], bordering_insee_codes: [], all_eligibility_codes: [offer.insee_code]) if [1,2,3].include?(i)
      hr.set(seeked_district_insee_codes: [], bordering_insee_codes: [offer.insee_code], all_eligibility_codes: [offer.insee_code]) if [4,5].include?(i)
    end 

    subject.perform_now(offer.id.to_s)
    offer.reload
    expect(offer.async_eligible_requests_nb).to eq(6)
    expect(offer.async_eligible_department_nb).to eq(6)
    expect(offer.async_eligible_seeked_nb).to eq(3)
    expect(offer.async_eligible_bordering_nb).to eq(2)
    expect(offer.async_eligible_seeked_bordering_nb).to eq(5)
    expect(offer.async_eligible_other_from_department_nb).to eq(1)
  end 

end
